--[[
@name Skillet TSM plugin.
@author Rustam SecondFry Gubaydullin [https://twitch.tv/second_fry]

The MIT License Copyright (c) 2020 Rustam SecondFry Gubaydullin

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES
OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
OR OTHER DEALINGS IN THE SOFTWARE.
]]--

--[[
  price = TSM_API.GetCustomPriceValue('dbmarket', `i:${itemID}`)

You may consider reading this:
--- Evalulates a custom price string or price source key for a given item
-- @within Price
-- @tparam string customPriceStr The custom price string or price source key to get the value of
-- @tparam string itemString The TSM item string to get the value for
-- @treturn number The value in copper or nil if the custom price string is not valid
-- @treturn string The (localized) error message if the custom price string is not valid or nil if it is valid
function TSM_API.GetCustomPriceValue(customPriceStr, itemString)

--- Gets a list of price source keys which can be used in TSM custom prices.
-- @within Price
-- @tparam table result A table to store the result in
-- @treturn table The passed table, populated with price source keys
function TSM_API.GetPriceSourceKeys(result)

]]--

Skillet.TSMPlugin = {}

local plugin = Skillet.TSMPlugin
local L = Skillet.L

plugin.options =
{
  type = 'group',
  name = 'TradeSkillMaster',
  order = 1,
  args = {
    enabled = {
      type = 'toggle',
      name = 'Enabled',
      get = function()
        return Skillet.db.profile.plugins.TSM.enabled
      end,
      set = function(_, value)
        Skillet.db.profile.plugins.TSM.enabled = value
        Skillet:UpdateTradeSkillWindow()
      end,
      width = 'double',
      order = 1
    },
    onlyPositive = {
      type = 'toggle',
      name = 'Only positive?',
      desc = 'Only show positive craft values?',
      get = function()
        return Skillet.db.profile.plugins.TSM.onlyPositive
      end,
      set = function(self,value)
        Skillet.db.profile.plugins.TSM.onlyPositive = value
        if value then
          Skillet.db.profile.plugins.TSM.onlyPositive = value
        end
        Skillet:UpdateTradeSkillWindow()
      end,
      width = 'double',
      order = 2
    },
    customPriceStrBasic = {
      type = 'select',
      name = 'Custom Price Source',
      desc = 'Custom Price Source to use from TSM',
      order = 3,
      get = function() 
        return Skillet.db.profile.plugins.TSM.customPriceStrBasic
      end,
      set = function(_, value)
        Skillet.db.profile.plugins.TSM.customPriceStrBasic = value
        Skillet:SortAndFilterRecipes()
        Skillet:UpdateTradeSkillWindow()
      end,
      values = {
        [' '] = ' ',
--          Additional entries filled in dynamically by RegisterTSMPriceSources
      },
    },
    useExtended = {
      type = 'toggle',
      name = 'Use extended?',
      desc = 'Should Skillet use following Material Cost Method and Expected Price Method strings?',
      get = function()
        return Skillet.db.profile.plugins.TSM.useExtended
      end,
      set = function(_, value)
        Skillet.db.profile.plugins.TSM.useExtended = value
        if value then
          Skillet.db.profile.plugins.TSM.useExtended = value
        end
        Skillet:SortAndFilterRecipes()
        Skillet:UpdateTradeSkillWindow()
      end,
      width = 'double',
      order = 4
    },
    methodExpectedPrice = {
      type = 'input',
      name = 'Expected Price Method',
      desc = 'Your expectation on how much item will sell for',
      width = 'double',
      order = 5,
      get = function() 
        return Skillet.db.profile.plugins.TSM.methodExpectedPrice
      end,
      set = function(_, value)
        Skillet.db.profile.plugins.TSM.methodExpectedPrice = value
        Skillet:SortAndFilterRecipes()
        Skillet:UpdateTradeSkillWindow()
      end
    },
    methodMaterialCost = {
      type = 'input',
      name = 'Material Cost Method',
      desc = 'TSM Material Cost Method string',
      width = 'double',
      order = 6,
      get = function() 
        return Skillet.db.profile.plugins.TSM.methodMaterialCost
      end,
      set = function(_, value)
        Skillet.db.profile.plugins.TSM.methodMaterialCost = value
        Skillet:SortAndFilterRecipes()
        Skillet:UpdateTradeSkillWindow()
      end
    },
  },
}

function plugin.OnInitialize()
  if not Skillet.db.profile.plugins.TSM then
    Skillet.db.profile.plugins.TSM = {
      ['enabled'] = true,
      ['customPriceStrBasic'] = 'DBMarket',
      ['useExtended'] = true,
      ['methodMaterialCost'] = 'matprice',
      ['methodExpectedPrice'] = '0.95 * first(dbminbuyout, dbmarket)'
    }
  end

  plugin.RegisterTSMPriceSources()
  Skillet:AddPluginOptions(plugin.options)
end

function plugin.RegisterTSMPriceSources()
  if not TSM_API then return end
  
  local res = {}
  local tmp = {}
  TSM_API.GetPriceSourceKeys(tmp)
  for _, v in ipairs(tmp) do
    res[v] = v
  end
  plugin.options.args.customPriceStrBasic.values = res
end

local function TSMQuery(source, itemString)
  return source, TSM_API.GetCustomPriceValue(source, itemString)
end

local function TSMCraftValue(itemID)
  if not TSM_API then return nil end

  local itemString = 'i:'..itemID
  local source, res

  if Skillet.db.profile.plugins.TSM.useExtended then
    source = 'Craft Value'
    _, res = TSMQuery(plugin.options.args.methodExpectedPrice.get(), itemString)
  else
    source, res = TSMQuery(plugin.options.args.customPriceStrBasic.get(), itemString)
  end
  
  if not res then source, res = TSMQuery('dbminbuyout', itemString) end
  if not res then source, res = TSMQuery('dbmarket', itemString) end
  return source, res
end

local function TSMMaterialCost(itemID)
  if not TSM_API then return nil end

  local itemString = 'i:'..itemID
  local source, res

  if Skillet.db.profile.plugins.TSM.useExtended then
    source = 'Material Cost'
    _, res = TSMQuery(plugin.options.args.methodMaterialCost.get(), itemString)
  else
    source, res = TSMQuery(plugin.options.args.customPriceStrBasic.get(), itemString)
  end
  
  if not res then source, res = TSMQuery('dbminbuyout', itemString) end
  if not res then source, res = TSMQuery('dbmarket', itemString) end
  return source, res
end

function plugin.GetExtraText(skill, recipe)
  if not Skillet.db.profile.plugins.TSM.enabled then return nil, nil end
  if not TSM_API then return nil, nil end
  if not skill or not recipe then return nil, nil end
  if not recipe.itemID then return nil, nil end
  
  local label, extra_text
  local itemID = recipe.itemID

  local source, price = TSMCraftValue(itemID)
  if not price then return nil, nil end

  extra_text = Skillet:FormatMoneyFull(price, true);
  label = source..':'
  return label, extra_text
end

function plugin.RecipeNameSuffix(skill, recipe)
  if not Skillet.db.profile.plugins.TSM.enabled then return nil end
  if not TSM_API then return nil end
  if not recipe then return nil end
  if not recipe.itemID then return nil end

  local text
  local itemID = recipe.itemID

  local value
  _, value = TSMCraftValue(itemID)
  if not value then return nil end

  value = value * recipe.numMade

  local matsum = 0
  for k,v in pairs(recipe.reagentData) do
    local iprice
    _, iprice = TSMMaterialCost(v.reagentID)

    if not iprice then
      matsum = value + 1
      break
    end

    matsum = matsum + v.numNeeded * iprice
  end
  value = value - matsum
  recipe.suffix = value

  if Skillet.db.profile.plugins.TSM.onlyPositive and value <= 0 then return nil end

  text = Skillet:FormatMoneyShort(value, true)
  return text
end

Skillet:RegisterRecipeNamePlugin('TSMPlugin')   -- we have a RecipeNamePrefix or a RecipeNameSuffix function

Skillet:RegisterDisplayDetailPlugin('TSMPlugin')  -- we have a GetExtraText function
